/*
 * cfgmgt.c
 * Thomas Nemeth, le 09.09.2000
 *
 *   Copyright (C) 1999  Thomas Nemeth
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <strings.h>
#include <errno.h>
#include <sys/stat.h>
#include "defines.h"
#include "my_strings.h"
#include "nsystem.h"
#include "cfgmgt.h"


void get_string_from_line (char **var, char *line) {
    char *result = NULL;
    char  tmpresult [MAXSTRLEN];
    int   i, j, s;

    j = 0;
    s = 0;
    for (i = 0 ; i < strlen (line) ; i++) {
        if (line [i] == '=') s = 1;
        if ( (s == 1) &&
             (line [i] != ' ') &&
             (line [i] != '=') &&
             (line [i] != '\n') ) {
            tmpresult [j] = line [i];
            j++;
        }
    }
    tmpresult [j] = 0;
    if (strlen (tmpresult) > 0) {
        MY_ALLOC (result, (strlen (tmpresult) + 1), char);
        strcpy (result, tmpresult);
        if (*var != NULL) free (*var);
        *var = result;
        #ifdef DEBUG_NOTIFY
            printf ("%s : %s\n", RETURN_VALUE, *var);
        #endif
    }
}

void get_int_from_line (int *var, char *line) {
    int  result = 0;
    char unusedstr [MAXSTRLEN];

    sscanf (line, "%s = %d", unusedstr, &result);
    *var = result;
}

int config_part (char *config_str) {
    char KEY [MAXSTRLEN];
    int  i, j;

    j = 0;
    for (i = 0 ; i < strlen (config_str) ; i++) {
        if (config_str[i] == '=') {
            KEY[j] = 0;
            break;
        } else if (config_str[i] != ' ') {
            KEY[j] = config_str[i];
            j++;
        }
    }
    #ifdef DEBUG_NOTIFY
        printf ("%s : %s\n", KEY_READ, KEY);
    #endif
    if (strcmp (KEY, KEY_MP3) == 0) return CFG_MP3PLY;
    if (strcmp (KEY, KEY_WAV) == 0) return CFG_WAVPLY;
    if (strcmp (KEY, KEY_ALR) == 0) return CFG_ALARMF;
    if (strcmp (KEY, KEY_NOT) == 0) return CFG_NOTIFF;
    if (strcmp (KEY, KEY_MSG) == 0) return CFG_LSTFIL;
    if (strcmp (KEY, KEY_NBS) == 0) return CFG_NSOUND;
    return CFG_UNKNWN;
}

void create_directories (char *Home) {
    DIR  *directory;
    char  configdir [MAXSTRLEN];
    char  options [MAXSTRLEN];
    int   code;

    sprintf (configdir, "%s/%s", Home, DEFAULT_DIRECTORY);
    
    if ( (directory = opendir (configdir) ) == NULL) {
        code = mkdir (configdir, S_IRUSR | S_IWUSR | S_IXUSR);
        if (code == -1) {
            perror (configdir);
            exit (1);
        }
        sprintf (configdir, "%s/%s/%s",
                 Home,
                 DEFAULT_DIRECTORY,
                 REMINDER_DIRECTORY);
        code = mkdir (configdir, S_IRUSR | S_IWUSR | S_IXUSR);
        if (code == -1) {
            perror (configdir);
            exit (1);
        }
        sprintf (options, "%s%s/*.wav %s/%s/",
                 INSTALL_DIRECTORY,
                 SYSTEMWIDE_CFG,
                 Home,
                 DEFAULT_DIRECTORY);
        my_system ("cp",options, VRAI, VRAI);
    } else {
        closedir (directory);
    }
}

void load_config () {
    char    *Home;
    char    *configfile;
    char     tmpstr [MAXSTRLEN + 1];
    FILE    *file;

    Home = getenv ("HOME");
    create_directories (Home);
    MY_ALLOC (configfile, (MAXSTRLEN + 1), char);
    sprintf(configfile, "%s/%s/%s", Home, DEFAULT_DIRECTORY, CONFIG_FILE);
    config.filename      = NULL;
    config.soundfile     = NULL;
    config.mode          = NOTH;
    config.command       = NON;
    config.nb_sound_play = NB_SOUND_PLAY;
    config.mp3_player    = NULL;
    config.wav_player    = NULL;
    config.alarm_sound   = NULL;
    config.notify_sound  = NULL;
    config.list_file     = NULL;
    config.list          = NULL;
    config.option        = NULL;
    if ( (file = fopen (configfile, "r") ) != NULL) {
        while (! feof (file) ) {
            fgets (tmpstr, (int) MAXSTRLEN, file);
            #ifdef DEBUG_NOTIFY
                printf ("%s : %s", LINE_READ, tmpstr);
            #endif
            if ( (tmpstr[0] != '#') && (tmpstr[0] != '\n') ) {
                switch (config_part (tmpstr) ) {
                    case CFG_MP3PLY :
                        get_string_from_line (&config.mp3_player,    tmpstr);
                        break;
                    case CFG_WAVPLY :
                        get_string_from_line (&config.wav_player,    tmpstr);
                        break;
                    case CFG_ALARMF :
                        get_string_from_line (&config.alarm_sound,   tmpstr);
                        break;
                    case CFG_NOTIFF :
                        get_string_from_line (&config.notify_sound,  tmpstr);
                        break;
                    case CFG_LSTFIL :
                        get_string_from_line (&config.list_file,     tmpstr);
                        break;
                    case CFG_NSOUND :
                        get_int_from_line    (&config.nb_sound_play, tmpstr);
                        break;
                    default :
                        printf ("%s :\n%s\n", UNRECONGIZED_CONFIG_ITEM, tmpstr);
                }
            }
        }
        #ifdef DEBUG_NOTIFY
            printf ("%s mp3_player    = %s\n", CONFIG_ITEM, config.mp3_player);
            printf ("%s wav_player    = %s\n", CONFIG_ITEM, config.wav_player);
            printf ("%s alarm_sound   = %s\n", CONFIG_ITEM, config.alarm_sound);
            printf ("%s notify_sound  = %s\n", CONFIG_ITEM, config.notify_sound);
            printf ("%s list_file     = %s\n", CONFIG_ITEM, config.list_file);
            printf ("%s nb_sound_play = %d\n", CONFIG_ITEM, config.nb_sound_play);
        #endif
        fclose (file);
    }
    free (configfile);
}

void update_config () {
    char *Home;
    FILE *list_file;

    Home = getenv ("HOME");
    if (config.filename == NULL) {
        MY_ALLOC (config.filename, MAXSTRLEN + 1, char);
        sprintf (config.filename, "%s/%s/%s",
                 Home,
                 DEFAULT_DIRECTORY,
                 MESSAGE_FILE);
    }
    if ( (config.soundfile == NULL) && (config.mode == COMD) ) {
        MY_ALLOC (config.soundfile, MAXSTRLEN + 1, char);
        if (config.alarm_sound == NULL) {
            sprintf (config.soundfile, "%s/%s/%s",
                     Home,
                     DEFAULT_DIRECTORY,
                     ALARM_SOUNDFILE);
        } else {
            if ( (config.alarm_sound[0] != '/') &&
                 (config.alarm_sound[0] != '~') )
                sprintf (config.soundfile, "%s/%s/%s",
                         Home,
                         DEFAULT_DIRECTORY,
                         config.alarm_sound);
            else
                sprintf (config.soundfile, "%s", config.alarm_sound);
        }
    }
    if ( (config.soundfile == NULL) && (config.mode == AUTO) ) {
        MY_ALLOC (config.soundfile, MAXSTRLEN + 1, char);
        if (config.notify_sound == NULL) {
            sprintf (config.soundfile, "%s/%s/%s",
                     Home,
                     DEFAULT_DIRECTORY,
                     NOTIFY_SOUNDFILE);
        } else {
            if ( (config.notify_sound[0] != '/') &&
                 (config.notify_sound[0] != '~') )
                sprintf (config.soundfile, "%s/%s/%s",
                         Home,
                         DEFAULT_DIRECTORY,
                         config.notify_sound);
            else
                sprintf (config.soundfile, "%s", config.notify_sound);
        }
    }
    if (config.list_file == NULL) {
        MY_ALLOC (config.list_file, MAXSTRLEN + 1, char);
        sprintf (config.list_file, "%s/%s/%s",
                 Home,
                 DEFAULT_DIRECTORY,
                 MESSAGES_LIST);
    }
    if ( (list_file = fopen (config.list_file, "r") ) != NULL) {
        FileList *p = NULL;
        int       i, j;
        char      tmpstr[MAXSTRLEN + 1];
        char      name[MAXSTRLEN + 1];
        tmpstr[0] = 0;
        fgets (tmpstr, (int) MAXSTRLEN, list_file);
        if (tmpstr[0] != 0) {
            MY_ALLOC (config.list, 1, FileList);
            p = config.list;
        }
        while (! feof (list_file) ) {
            j = 0;
            for (i = 0 ; i < strlen (tmpstr) ; i++) {
                if (tmpstr[i] != '\n') name[j++] = tmpstr[i];
            }
            name[j]=0;
            #ifdef DEBUG_NOTIFY
                printf ("%s :  \"%s\"\n", LINE_READ, tmpstr);
                printf ("%s -> \"%s\"\n", LINE_READ, name);
            #endif
            if ( (strlen (tmpstr) != 0) && (tmpstr[0] != '\n') ) {
                MY_ALLOC (p->filename, MAXSTRLEN + 1, char);
                MY_ALLOC (p->name, strlen (name) + 1, char);
                sprintf (p->filename, "%s/%s/%s/%s",
                         Home,
                         DEFAULT_DIRECTORY,
                         REMINDER_DIRECTORY,
                         name);
                sprintf (p->name, "%s", name);
            } else {
                p->filename = NULL;
                p->name     = NULL;
            }
            p->next = NULL;
            if (! feof (list_file) ) {
                tmpstr[0] = 0;
                fgets (tmpstr, (int) MAXSTRLEN, list_file);
                if ( (strlen (tmpstr) != 0) && (tmpstr[0] != '\n') ) {
                    MY_ALLOC (p->next, 1, FileList);
                    p = p->next;
                }
            }
        }
    }
}

void free_config () {
    if (config.mp3_player   != NULL) free (config.mp3_player);
    if (config.wav_player   != NULL) free (config.wav_player);
    if (config.alarm_sound  != NULL) free (config.alarm_sound);
    if (config.notify_sound != NULL) free (config.notify_sound);
    if (config.filename     != NULL) free (config.filename);
    if (config.soundfile    != NULL) free (config.soundfile);
    if (config.list_file    != NULL) free (config.list_file);
    if (config.option       != NULL) free (config.option);
    if (config.list != NULL) {
        FileList *q, *p = config.list;
        while (p != NULL) {
            q = p;
            p = p->next;
            free(q->filename);
            free(q->name);
            free(q);
        }
    }
}
