/*
 * commandline.c
 * Thomas Nemeth, le 09.09.2000
 *
 *   Copyright (C) 1999  Thomas Nemeth
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <ctype.h>
#include "defines.h"
#include "my_strings.h"
#include "commandline.h"
#include "cfgmgt.h"


/* ====== FONCTIONS ======================================================= */

char *get_month_ni (int month) {
    char *MONTHS[12] = { JAN, FEB, MAR, APR,
                         MAY, JUN, JUL, AUG,
                         SEP, OCT, NOV, DEC };
    char *text = NULL;
    if(month>=0 && month<=11) {
        MY_ALLOC (text, 4, char);
        strcpy(text,MONTHS [month]);
    } else {
        printf("err. get_month_ni month=%d\n",month);
        text="";
    }
    return text;
}

char *get_month_n (const char *date) {
    char  tm [3];
    int   m = 0;
    char *text = NULL;

    if (date != NULL) {
        tm [0] = date [4];
        tm [1] = date [5];
        tm [2] = 0;
        m = atoi (tm);
        text = get_month_ni (m - 1);
    }
    return text;
}

char *get_year (const char *date) {
    char *y = NULL;

    if (date != NULL) {
        MY_ALLOC (y, 5, char);
        y [0] = date [0];
        y [1] = date [1];
        y [2] = date [2];
        y [3] = date [3];
        y [4] = 0;
    }
    return y;
}

void Usage (int code) {
    if (code == 2) printf ("%s\n\n", BAD_ARG_NUM);
    printf ("%s\n", USAGE1);
    printf ("%s\n", USAGE2);
    printf ("%s\n", USAGE3);
    printf ("%s\n", USAGE4);
    printf ("%s\n", USAGE5);
    printf ("%s\n", USAGE6);
    printf ("%s\n", USAGE7);
    printf ("%s\n", USAGE8);
    printf ("%s\n", USAGE9);
    printf ("%s\n", USAGE10);
    printf ("%s\n", USAGE11);
    printf ("%s\n", USAGE12);
    printf ("%s\n", USAGE13);
    printf ("%s\n", USAGE14);
    printf ("%s\n", USAGE15);
    printf ("%s\n", USAGE16);
    printf ("%s\n", USAGE17);
    printf ("%s\n", USAGE18);
    printf ("\n");
    free_config ();
    exit (code);
}

void fatal_error () {
    printf ("%s\n", TOO_MUCH_ARGS);
    free_config ();
    exit (1);
}

void traite_argv (int argc, char* argv[]) {
    int   ind = 1;
    char *month;
    char *year;
    char *Home;

    Home = getenv ("HOME");
    month = get_month_n (COMPILATION_DATE);
    year  = get_year    (COMPILATION_DATE);

    while (ind < argc) {
        if (argv[ind][0] == '-') {
            switch (argv[ind][1]) {
                case 'h' :
                    printf ("notify (Thomas Nemeth - V %s - %s %s)\n",
                            VERSION,
                            month,
                            year);
                    Usage  (0);
                    break;
                case 'v' :
                    printf ("notify (Thomas Nemeth - V %s - %s %s)\n",
                            VERSION,
                            month,
                            year);
                    exit (0);
                    break;
                case 'q' :
                    if ( (config.mode != NOTH) &&
                         (config.mode != COMD) &&
                         (config.mode != AUTO) &&
                         (config.mode != TEMP) )
                        fatal_error ();
                    config.nb_sound_play = 0;
                    if (config.mode == NOTH) config.mode = TEMP;
                    break;
                case 'b' :
                    if ( (config.mode != NOTH) &&
                         (config.mode != COMD) &&
                         (config.mode != AUTO) &&
                         (config.mode != TEMP) )
                        fatal_error ();
                    ind++;
                    if (ind == argc) Usage (2);
                    MY_ALLOC (config.soundfile, strlen (argv[ind]) + 1, char);
                    strcpy (config.soundfile, argv[ind]);
                    if (config.mode == NOTH) config.mode = TEMP;
                    break;
                case 'n' :
                    if ( (config.mode != NOTH) &&
                         (config.mode != COMD) &&
                         (config.mode != AUTO) &&
                         (config.mode != TEMP) )
                        fatal_error ();
                    ind++;
                    if (ind == argc) Usage (2);
                    if (isdigit (argv[ind][0]) )
                        config.nb_sound_play = atoi (argv[ind]);
                    if (config.mode == NOTH) config.mode = TEMP;
                    break;
                case 'f' :
                    if ( (config.mode != NOTH) &&
                         (config.mode != COMD) &&
                         (config.mode != TEMP) )
                        fatal_error ();
                    ind++;
                    if (ind == argc) Usage (2);
                    MY_ALLOC (config.filename, strlen (argv[ind]) + 1, char);
                    strcpy (config.filename, argv[ind]);
                    config.command = FIL;
                    config.mode    = COMD;
                    break;
                case 'm' :
                    if ( (config.mode != NOTH) &&
                         (config.mode != COMD) &&
                         (config.mode != TEMP) )
                        fatal_error ();
                    ind++;
                    if (ind == argc) Usage (2);
                    MY_ALLOC (config.filename, strlen (argv[ind]) + 1, char);
                    strcpy (config.filename, argv[ind]);
                    config.command = MSG;
                    config.mode    = COMD;
                    break;
                case 'a' :
                    if (config.mode != NOTH)
                        fatal_error ();
                    ind++;
                    if (ind == argc) Usage (2);
                    MY_ALLOC (config.option, strlen (argv[ind]) + 1, char);
                    strcpy (config.option, argv[ind]);
                    ind++;
                    if (ind == argc) Usage (2);
                    MY_ALLOC (config.filename, strlen (argv[ind]) + 1, char);
                    strcpy (config.filename, argv[ind]);
                    config.command = ADD;
                    config.mode    = MNGT;
                    break;
                case 'l' :
                    if (config.mode != NOTH)
                        fatal_error ();
                    config.command = LST;
                    config.mode    = MNGT;
                    break;
                case 'd' :
                    if (config.mode != NOTH)
                        fatal_error ();
                    ind++;
                    if (ind == argc) Usage (2);
                    MY_ALLOC (config.filename, strlen (argv[ind]) + 1, char);
                    strcpy (config.filename, argv[ind]);
                    config.command = DEL;
                    config.mode    = MNGT;
                    break;
                case 's' :
                    if ( (config.mode != NOTH) &&
                         (config.mode != AUTO) &&
                         (config.mode != TEMP) )
                        fatal_error ();
                    ind++;
                    if (ind == argc) Usage (2);
                    if (isdigit (argv[ind][0]) )
                        config.command = atoi (argv[ind]);
                    config.mode = AUTO;
                    break;
                 default:
                    printf ("%s : %s\n", UNKNOWN_OPTION, argv[ind]);
                    Usage  (1);
                    break;
            }
        } else {
            printf ("%s : %s\n", UNKNOWN_OPTION, argv[ind]);
            Usage  (1);
        }
        ind++;
    }
    free (month);
    free (year);
    if (config.mode == NOTH) config.mode = COMD;
}
