/*
 * graphics.c
 * Thomas Nemeth, le 09.09.2000
 *
 *   Copyright (C) 1999  Thomas Nemeth
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <gtk/gtk.h>
#include "defines.h"
#include "my_strings.h"


/* ====== FONCTIONS ======================================================= */

void load_message (GtkWidget *vbox, const char *filename) {
    FILE      *file;
    GtkWidget *texte;
    GtkWidget *hbox;
    char      *line = NULL;
    char      *text = NULL;
    int        i, j;
    GdkColor   blanc   = { 0, 65535, 65535, 65535};
    GtkStyle  *style;

    MY_ALLOC (line,     MAXSTRLEN + 1, char);
    MY_ALLOC (text,     MAXSTRLEN + 1, char);
    if ( (file = fopen (filename, "r") ) == NULL) {
        #ifdef DEBUG_NOTIFY
            printf ("%s : %s\n", ERROR, filename);
        #endif
        hbox  = gtk_hbox_new (FALSE, 0);
        texte = gtk_label_new (OPEN_ERROR);
        gtk_label_set_justify (GTK_LABEL (texte), GTK_JUSTIFY_LEFT);
        gtk_box_pack_start (GTK_BOX (hbox), texte, FALSE, TRUE, 0);
        gtk_widget_show (texte);
        gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
        gtk_widget_show (hbox);
        hbox  = gtk_hbox_new (FALSE, 0);
        texte = gtk_label_new (filename);
        gtk_label_set_justify (GTK_LABEL (texte), GTK_JUSTIFY_LEFT);
        gtk_box_pack_start (GTK_BOX (hbox), texte, FALSE, TRUE, 0);
        gtk_widget_show (texte);
        gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
        gtk_widget_show (hbox);
    } else {
        gtk_widget_set_name (vbox, filename);
        fgets (line, (int) MAXSTRLEN, file);
        #ifdef DEBUG_NOTIFY
            printf ("%s :  \"%s\"\n", LINE_READ, line);
        #endif
        while (! feof (file) ) {
            j = 0;
            for (i = 0 ; i < strlen (line) ; i++) {
                if (line[i] != '\n') text[j++] = line[i];
            }
            text[j]=0;
            #ifdef DEBUG_NOTIFY 
                printf ("%s -> \"%s\"\n", LINE_READ, text);
            #endif
            hbox  = gtk_hbox_new (FALSE, 0);
            gtk_widget_show (hbox);
            texte = gtk_label_new (text);
    /* CHANGEMENT DE COULEUR : FOND BLANC ! */
    style = gtk_style_copy (gtk_widget_get_default_style () );
    for (i = 0 ; i < 5 ; i++) {
        style->bg[i]   = blanc;
        style->base[i] = blanc;
    }
    gtk_widget_set_style (texte, style);
    gtk_widget_set_style (hbox, style);
    gtk_widget_set_style (vbox, style);
    /* ------> CHANGEMENT DE COULEUR : FOND BLANC ! */
            gtk_label_set_justify (GTK_LABEL (texte), GTK_JUSTIFY_LEFT);
            gtk_box_pack_start (GTK_BOX (hbox), texte, FALSE, FALSE, 0);
            gtk_widget_show (texte);
            gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
            gtk_widget_show (hbox);
            fgets (line, (int) MAXSTRLEN, file);
            if (! feof (file) ) {
                #ifdef DEBUG_NOTIFY
                    printf ("%s :  \"%s\"\n", LINE_READ, line);
                #endif
            }
        }
        fclose (file);
    }
    free (line);
    free (text);
}

void add_title (GtkWidget *vbox, const char *filename, int search) {
    FileList  *p = config.list;
    GtkWidget *hsep;
    GtkWidget *hbox;
    GtkWidget *texte;

    if (search == VRAI)
        while ( (p != NULL) && (strcmp (p->filename, filename) != 0) ) {
            p = p->next;
        }
    if ( ( (p != NULL) && (p->filename != NULL) ) ||
         ( (search == VRAI) && (filename != NULL) ) ) {
        hsep = gtk_hseparator_new ();
        gtk_box_pack_start (GTK_BOX (vbox), hsep, FALSE, FALSE, 0);
        gtk_widget_show (hsep);

        hbox  = gtk_hbox_new (FALSE, 0);
        gtk_widget_show (hbox);
        if ( (p != NULL) && (p->filename != NULL) )
            texte = gtk_label_new (p->name);
        else
            texte = gtk_label_new (filename);
        gtk_label_set_justify (GTK_LABEL (texte), GTK_JUSTIFY_LEFT);
        gtk_box_pack_start (GTK_BOX (hbox), texte, FALSE, FALSE, 0);
        gtk_widget_show (texte);
        gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
        gtk_widget_show (hbox);

        hsep = gtk_hseparator_new ();
        gtk_box_pack_start (GTK_BOX (vbox), hsep, FALSE, FALSE, 0);
        gtk_widget_show (hsep);
    }
}

void load_messages (GtkWidget *vbox) {
    FileList  *p = config.list;

    while (p != NULL) {
        if (p->filename != NULL) {
            add_title (vbox, p->name, FAUX);
            load_message (vbox, p->filename);
        }
        p = p->next;
    }
}

void quit_cb (GtkWidget *widget, void *data) {
    gtk_main_quit ();
}

void close_cb (GtkWidget *widget, void *data) {
    GtkWidget *vbox = GTK_WIDGET (data);
    GList     *children;
    char      *filename;
    FileList  *p = config.list;

    if ( (config.mode == AUTO) &&
         ( (config.command == OP2) ||
           (config.command == OP4) ) ) {
        children = gtk_container_children (GTK_CONTAINER (vbox) );
        if (children != NULL)
            g_list_foreach (children,
                            GTK_SIGNAL_FUNC (gtk_widget_destroy),
                            NULL);
        filename = gtk_widget_get_name (vbox);
        while ( (p != NULL) && (strcmp (filename, p->filename) != 0) ) {
            p = p->next;
        }
        p = p->next;
        if (p != NULL) {
            if (config.command == OP4) add_title (vbox, p->name, FAUX);
            load_message (vbox, p->filename);
        } else
            gtk_main_quit ();
    } else
        gtk_main_quit ();
}

void window (const char *filename) {
    GtkWidget *application;
    GtkWidget *main_vbox;
    GtkWidget *vbox;
    GtkWidget *scrolled_window;
    GtkWidget *bouton;
    GdkColor   blanc   = { 0, 65535, 65535, 65535};
    GtkStyle  *style;
    int        i;

    /* fentre principale */
    application = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (application), "notify");

    /* boite principale */
    main_vbox = gtk_vbox_new (FALSE, 1);
    gtk_container_border_width (GTK_CONTAINER (main_vbox), 6);
    gtk_container_add (GTK_CONTAINER (application), main_vbox);
    gtk_widget_show (main_vbox);

    /* vnements de fermeture */
    gtk_signal_connect (GTK_OBJECT (application), "delete_event",
                        GTK_SIGNAL_FUNC (quit_cb), NULL);
    gtk_signal_connect (GTK_OBJECT (application), "destroy", 
		                GTK_SIGNAL_FUNC (quit_cb), 
		                "WM destroy");

    /* fentre avec ascenseur */
    scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC,
                                    GTK_POLICY_ALWAYS);

    /* boite pour afficher le texte */
    vbox  = gtk_vbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
    /* CHANGEMENT DE COULEUR : FOND BLANC ! */
    style = gtk_style_copy (gtk_widget_get_default_style () );
    for (i = 0 ; i < 5 ; i++) {
        style->bg[i]   = blanc;
        style->base[i] = blanc;
    }
    gtk_widget_set_style (vbox, style);
    /* ------> CHANGEMENT DE COULEUR : FOND BLANC ! */
    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW
                                           (scrolled_window), vbox);

    if ( (filename != NULL) && (filename[0] != 0) ) {
        if (config.command == OP4) add_title (vbox, filename, VRAI);
        load_message (vbox, filename);
    } else
        load_messages (vbox);

    gtk_widget_show (vbox);
    /* ------ boite pour afficher le texte */


    gtk_box_pack_start (GTK_BOX (main_vbox), scrolled_window, TRUE, TRUE, 6);
    gtk_widget_show (scrolled_window);
    /* ------ fin fentre assenceur */

    /* bouton de fermeture */
    bouton = gtk_button_new_with_label (CLOSE);
    gtk_signal_connect (GTK_OBJECT (bouton), "clicked",
                        GTK_SIGNAL_FUNC (close_cb),
                        vbox);

    gtk_box_pack_start (GTK_BOX (main_vbox), 
                        bouton, FALSE, FALSE, 0);

    GTK_WIDGET_SET_FLAGS (GTK_WIDGET (bouton), GTK_CAN_DEFAULT);
    gtk_widget_grab_default (GTK_WIDGET (bouton) );
    gtk_widget_draw_default (GTK_WIDGET (bouton) );

    gtk_widget_show (bouton);
    /* ------ fin bouton de fermeture */

    /* taille et lancement */
    gtk_widget_set_usize (GTK_WIDGET (application), 600, 300);

    gtk_widget_show (application);

    gtk_main ();
}
