/*
 * notify.c
 * Thomas Nemeth, le 08.09.2000
 *
 *   Copyright (C) 1999  Thomas Nemeth
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <ctype.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <gtk/gtk.h>
#include "defines.h"
#include "my_strings.h"
#include "notify.h"
#include "nsystem.h"
#include "commandline.h"
#include "cfgmgt.h"
#include "graphics.h"


/* ====== FONCTIONS ======================================================= */

int command () {
    int val = 0;
    int exit_code = 0;

    play_sound (config.soundfile);
    if (config.command == MSG) {
        if (isdigit (config.filename[0]) ) val = atoi (config.filename);
        if (config.list != NULL) {
            FileList *p = config.list;
            FileList *q = NULL;
            FileList *f = NULL;
            FileList *e = NULL;
            int       i = 1;
            while (p != NULL) {
                if ( ( (val != 0) && (i == val) ) ||
                     ( (val == 0) &&
                       (strcmp (p->name, config.filename) == 0)
                     )
                   ) {
                   f = p;
                   e = q;
                }
                i++;
                q = p;
                p = p->next;
            }
            if (f != NULL) {
                free (config.filename);
                MY_ALLOC (config.filename, strlen (f->filename) + 1, char);
                strcpy (config.filename, f->filename);
            } else {
                printf ("   %s : %s\n", config.filename, NO_MESSAGE_FOUND);
                exit_code = 1;
            }
        } else {
            printf ("   %s\n", NO_MESSAGE);
            exit_code = 1;
        }
    }

    if (exit_code == 0) window (config.filename);
    return exit_code;
}

int manage () {
    FILE     *file;
    int       exit_code = 0;
    char     *Home;
    char      option[MAXSTRLEN];
    int       val = 0;

    Home = getenv ("HOME");
    switch (config.command) {
        case ADD :
            printf ("\n%s : %s\n", ADD_LABEL, config.option);
            sprintf (option, "%s %s/%s/%s/%s",
                     config.filename,
                     Home,
                     DEFAULT_DIRECTORY,
                     REMINDER_DIRECTORY,
                     config.option);
            my_system ("cp", option, FAUX, FAUX);
            if ( (file = fopen (config.list_file, "a") ) != NULL) {
                fprintf (file, "%s\n", config.option);
                fclose (file);
            } else {
                perror (config.list_file);
                exit_code = 1;
            }
            break;
        case LST :
            printf ("\n%s :\n", LIST_LABEL);
            underscore (LIST_LABEL, DOUBLE);
            if (config.list != NULL) {
                FileList *p = config.list;
                int       i = 1;
                while (p != NULL) {
                    if (p->name != NULL)
                        printf ("%s %d : %s\n", MESSAGE_DISP, i, p->name);
                    i++;
                    p = p->next;
                }
            } else printf ("   %s\n", NO_MESSAGE);
            printf ("\n");
            break;
        case DEL :
            printf ("\n%s :\n", DELETE_LABEL);
            underscore (DELETE_LABEL, DOUBLE);
            if (isdigit (config.filename[0]) ) val = atoi (config.filename);
            if (config.list != NULL) {
                FileList *p = config.list;
                FileList *q = NULL;
                FileList *f = NULL;
                FileList *e = NULL;
                int       i = 1;
                while (p != NULL) {
                    if ( ( (val != 0) && (i == val) ) ||
                         ( (val == 0) &&
                           (strcmp (p->name, config.filename) == 0)
                         )
                       ) {
                       f = p;
                       e = q;
                    }
                    i++;
                    q = p;
                    p = p->next;
                }
                if (f != NULL) {
                    my_system ("rm", f->filename, FAUX, FAUX);
                    if (e != NULL)
                        e->next = f->next;
                    else
                        config.list = f->next;
                    free (f->filename);
                    free (f->name);
                    free (f);
                    if ( (file = fopen (config.list_file, "w") ) != NULL) {
                        p = config.list;
                        while (p != NULL) {
                            if (p->name != NULL) fprintf (file, "%s\n", p->name);
                            p = p->next;
                        }
                        fclose (file);
                    } else {
                        perror (config.list_file);
                        exit_code = 1;
                    }
                } else
                    printf ("   %s : %s\n", config.filename, NO_MESSAGE_FOUND);
            } else printf ("   %s\n", NO_MESSAGE);
            printf ("\n");
    }
    return exit_code;
}

int automatic () {
    int   exit_code = 0;
    char *Home;

    Home = getenv ("HOME");
    switch (config.command) {
        case OP1 :
            if (config.list != NULL) {
                FileList *p = config.list;
                char      options [MAXSTRLEN * 2];
                char      tmp_file [MAXSTRLEN];
                sprintf (tmp_file, "%s/%s/%s",
                         Home,
                         DEFAULT_DIRECTORY,
                         DEFAULT_TMPFILE);
                strcpy (options, " ");
                while (p != NULL) {
                    if (p->filename != NULL) {
                        strcat (options, p->filename);
                        strcat (options, " ");
                    }
                    p = p->next;
                }
                strcat (options, "> ");
                strcat (options, tmp_file);
                #ifdef DEBUG_NOTIFY
                    printf ("options : %s\n", options);
                #endif
                my_system ("cat", options, FAUX, FAUX);
                play_sound (config.soundfile);
                window (tmp_file);
                my_system ("rm", tmp_file, FAUX, FAUX);
            } else {
                printf ("   %s\n", NO_MESSAGE);
                exit_code = 1;
            }
            break;
        case OP2 :
        case OP4 :
            if (config.list != NULL) {
                play_sound (config.soundfile);
                window (config.list->filename);
            } else {
                printf ("   %s\n", NO_MESSAGE);
                exit_code = 1;
            }
            break;
        case OP3 :
            if (config.list != NULL) {
                play_sound (config.soundfile);
                window (NULL);
            } else {
                printf ("   %s\n", NO_MESSAGE);
                exit_code = 1;
            }
            break;
        default :
            printf ("%s\n", AUTO_OPTIONS);
            exit_code = 1;
    }
    return exit_code;
}


/* ====== MAIN ============================================================ */

int main (int argc, char *argv[]) {
    int exit_code = 0;
    #ifdef USE_FORK
    int pid;
    int status;
    #endif

    /* initialisation */
    load_config ();
    gtk_init (&argc, &argv);
    traite_argv (argc, argv);
    update_config ();

    /* lancement des fonctions */
    switch (config.mode) {
        case COMD :
            exit_code = command ();
            break;
        case MNGT :
            exit_code = manage ();
            break;
        case AUTO :
            exit_code = automatic ();
            break;
        default :
            Usage (1);
    }

    /* libration mmoire et arrt */
    free_config ();

    /* Suppression des processus fils */
    #ifdef USE_FORK
        pid = waitpid (-1, &status, WNOHANG);
        #ifdef DEBUG_NOTIFY
            printf ("pid du fils : %d\n", pid);
        #endif
        #ifdef SIGNAL_HANDLING
            kill (pid, SIGUSR1);
        #else
            kill (pid, SIGTERM);
        #endif
    #endif

    return exit_code; 
}
