/*
 * nsystem.c
 * Thomas Nemeth, le 08.09.2000
 *
 *   Copyright (C) 1999  Thomas Nemeth
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include "defines.h"
#include "my_strings.h"
#include "nsystem.h"


/* ====== FONCTIONS ======================================================= */

extern char **environ;

int my_system (const char *command, const char *options, int redir, int bg) {
    int pid, status;

    if (command == 0)
        return 1;
    pid = fork ();
    if (pid == -1)
        return -1;
    if (pid == 0) {
        char *cmd;
        char *argv[4];
        MY_ALLOC (cmd, strlen (command) + strlen (options) + 2, char);
        strcpy (cmd, command);
        strcat (cmd, " ");
        strcat (cmd, options);
        if (redir == VRAI) strcat (cmd, " > /dev/null 2>&1");
        if (bg    == VRAI) strcat (cmd, " &");
        #ifdef DEBUG_NOTIFY
            printf ("%s : %s\n", SYSTEM_CMD, cmd);
        #endif
        argv[0] = "sh";
        argv[1] = "-c";
        argv[2] = cmd;
        argv[3] = 0;
        execve ("/bin/sh", argv, environ);
        free (cmd);
        exit (127);
    }
    do {
        if (waitpid (pid, &status, 0) == -1) {
            if (errno != EINTR)
                return -1;
        } else
            return status;
    } while (1);
}

#ifdef SIGNAL_HANDLING
void process_quit (int sig) {
    exit (0);
}
#endif

void play_sound (const char *filename) {
    int   redir = VRAI;
    int   bg    = VRAI;
    int   i;
    #ifdef USE_FORK
    pid_t            pid;
      #ifdef SIGNAL_HANDLING
        sigset_t         ens;
        struct sigaction action;
      #endif
    #endif

    #ifdef DEBUG_NOTIFY
        redir = FAUX;
    #endif
    #ifdef USE_FORK
    bg = FAUX;
    switch (pid = fork () ) {
        case (pid_t) -1 :
            printf ("%s\n", COULDNT_FORK);
        case (pid_t)  0 :
    #endif
            for (i = 0 ; i < config.nb_sound_play ; i++) {
                if (strstr (filename, ".wav") != NULL)
                    my_system (WAV_PLAYER, filename, redir, bg);
                if (strstr (filename, ".mp3") != NULL)
                    my_system (MP3_PLAYER, filename, redir, bg);
            }
    #ifdef USE_FORK
            #ifdef SIGNAL_HANDLING
                sigemptyset (&ens);
                sigaddset (&ens, SIGUSR1);
                action.sa_handler = process_quit;
                sigaction (SIGUSR1, &action, NULL);
                sigsuspend (&ens);
                exit (0);
            #else
                while (1) { }
            #endif
    }
    #endif
}

void underscore (const char *text, int form) {
    int i;

    for (i = 0 ; i < strlen (text) + 3 ; i++)
        if (form == DOUBLE)
            printf ("=");
        else
            printf ("-");
    printf ("\n\n");
}
